/**
  ******************************************************************************
  * @file           : main.c
  * @author         ：SINOMCU-AE
  * @brief          : Main program body
  *    
  *          This file provides example code for Timer14 input capture measure MCO 
  * or PA4 inputand printf by uart.system clock 48M for PLL out.LED1 blink by sysTick
  * soft timer,defualt blink period 1S;uart send information  of running count(redirect printf).
  * For details, see "readme.txt"   
  *         
  ******************************************************************************
   * @attention 
  *                           DISCLAIMER
  * The present firmware which is for guidance only aims at providing customers
  * with coding information regarding their products in order for them to save
  * time. As a result, SinoMCU shall not be held liable for any direct,indirect
  * or consequential damages with respect to any claims arising from the content 
  * of such firmware and/or the use made by customers of the coding information 
  * contained herein in connection with their products.
  *
  * <h2><center>&copy; Copyright (c) 2021 sinomcu All rights reserved.</center></h2>
  *
  ******************************************************************************
  */  
  
#include "system_define.h"

// LED blink half cycle in ms  
#define LED_BLINK_HALF_PER        500       // unit ms

FlagStatus TIME_CC_TRG_FLAG = RESET;        // flag for tiemr CC interrupt occur
uint16_t g_Signal_Freq_Hz=0,g_Signal_Duty_Percent=0;

/**
  * @brief  Main program
  * @param  None
  * @retval None
  */
int main(void) 
{
  uint32_t count=0;

  SystemClock_Config();                     // sysclk =PLL out 48MHz.use HSE,if HSE not oK,then use HSI instead of HSE
  SystemCoreClockUpdate();                  // get System Core Clock
    
  sys_tick_init();                          // system tick initializtion,defualt set 1ms
  gpio_init();                              // LED1 out,default low
#if(defined(EN_PRINT_UART) && EN_PRINT_UART)  
  MS32_RCC_SetUSARTClockSource(MS32_RCC_USART1_CLKSOURCE_HSI);  // uart clk HSI 8M     
  usart1_uart_init(115200);
#endif
  // user_config.h TIM14_IC_SIGNAL_SOURCE,0:LSI from MCO,PA8 GPIO,1:LSI from MCO,PA8 MCO,2:Timer14 IC PA4,PA8 MCO,PB7 PWM out
#if(TIM14_IC_SIGNAL_SOURCE==0)              // LSI from MCO,PA8 GPIO
  MS32_RCC_ConfigMCO(MS32_RCC_MCO_SOURCE_LSI, MS32_RCC_MCO_DIV1);    // no pin config,PA8 GPIO
#else                                       // LSI from MCO,PA8 MCO;Timer14 IC PA4,PA8 MCO,PB7 PWM out
  MS32_RCC_MCOConfig(MS32_RCC_MCO_SOURCE_LSI, MS32_RCC_MCO_DIV1);    // had finish pin config
#endif

#if(TIM14_IC_SIGNAL_SOURCE==2)              // Timer14 IC PA4,PA8 MCO,PB7 PWM out
  timer17_pwm_init();
#endif
  timer14_capture_init();
  LED1_ON();  
  
  printf("\r\n\r\n*****TIM14 LSI MCO Example*****");
  printf("\r\n--Inf:System Core Clock %d Hz.",SystemCoreClock);

  if( MS32_RCC_PLL_GetMainSource() == MS32_RCC_PLLSOURCE_HSE) {      // HSE to PLL
    printf("\r\n--Inf:HSE to PLL,PLL out for system clock.");
  } else if ( MS32_RCC_PLL_GetMainSource() == MS32_RCC_PLLSOURCE_HSI_DIV_2) {  // HSI to PLL
    printf("\r\n!-Inf:HSI to PLL,PLL out for system clock.");
  } else {
    printf("\r\n!!Inf:HSI for system clock.");                       // use HSI
  } 
  // user_config.h TIM14_IC_SIGNAL_SOURCE,0:LSI from MCO,PA8 GPIO,1:LSI from MCO,PA8 MCO,2:Timer14 IC PA4,PA8 MCO,PB7 PWM out
#if(TIM14_IC_SIGNAL_SOURCE==0)              // LSI from MCO,PA8 GPIO
  printf("\r\nTimer14 MCO LSI internal connect,PA8 GPIO."); 
#elif(TIM14_IC_SIGNAL_SOURCE==1)            // LSI from MCO,PA8 MCO;
  printf("\r\nTimer14 MCO LSI internal connect,PA8 MCO."); 
#elif(TIM14_IC_SIGNAL_SOURCE==2)            // Timer14 IC PA4,PA8 MCO,PB7 PWM out
  printf("\r\nTimer14 IC PA4,PA8 MCO,,PB7 PWM out.user need connect PA8 or PB7 to PA4."); 
#endif

  Start_100MS_SF_Timer();
  Start_User_MS_SF_Timer(LED_BLINK_HALF_PER);
  while(1) 
  {
    if(Get_State_100MS_SF_Timer() == SF_TIMER_OV) {
      Start_100MS_SF_Timer();
      if(TIME_CC_TRG_FLAG == SET) {         // Timer CC interrupt
        TIME_CC_TRG_FLAG = RESET;
        printf("\r\n--Inf:Signal Freq,Duty:%dHz,%d%%",g_Signal_Freq_Hz,g_Signal_Duty_Percent); 
      }
    }   
    
    if(Get_State_User_MS_SF_Timer() == SF_TIMER_OV) {
      Start_User_MS_SF_Timer(LED_BLINK_HALF_PER);// clear soft timer ov
      LED1_TOGGLE();
      count++;
      printf("\r\n--Inf: running count:%d",count);
    }
  }
}

/**
  * @brief  timer14 CC interrupt callback function
  * @param  CC1_value:timer input capture vale 
  * @retval None
  */
void Timer14_CC1_Callback(uint32_t CC1_value) {
  static TIM_IC_STATE Timer_IC_Cur_State =WAIT_FIRST_CAPTURE;
  static uint32_t first_ic_value=0,second_ic_value=0;
  uint16_t freq_original_value=0;
  
  switch(Timer_IC_Cur_State) {
    case WAIT_FIRST_CAPTURE:                     // a) previous state:wait for first capture
      first_ic_value =CC1_value;                 // rising edge occur
      MS32_TIM_IC_SetPolarity(TIM14, MS32_TIM_CHANNEL_CH1, MS32_TIM_IC_POLARITY_FALLING);
      Timer_IC_Cur_State =FIRST_RISE_CAPTURE;    // had first capture rising edge,to next state,need second capture falling edge
      break;
    case FIRST_RISE_CAPTURE:                     // b) previous state:first captures rising edge;for duty cycle calculation
      second_ic_value =CC1_value;
      MS32_TIM_IC_SetPolarity(TIM14, MS32_TIM_CHANNEL_CH1, MS32_TIM_IC_POLARITY_RISING); 
      Timer_IC_Cur_State =SECOND_FALL_CAPTURE;   // had second capture falling edge,to next state,need third capture rising edge
      break;
    case SECOND_FALL_CAPTURE:                    // c) previous state:second capture falling edge
      if(CC1_value > first_ic_value)             // no UIF occur
        freq_original_value =(CC1_value - first_ic_value);
      else                                       // UIF occur
        freq_original_value =(0xFFFF -first_ic_value + CC1_value);

      g_Signal_Freq_Hz=(uint16_t)(TIMER_CNT_CLOCK/(freq_original_value));      // calculation frequency

      if(second_ic_value > first_ic_value)       // no UIF occur,calculation duty cycle,percent
        g_Signal_Duty_Percent =(uint16_t)(((second_ic_value - first_ic_value)*100 / freq_original_value));
      else                                       // UIF occur,calculation duty cycle,percent
        g_Signal_Duty_Percent =(uint16_t)(((0xFFFF -first_ic_value +second_ic_value )*100 / freq_original_value));
        
      Timer_IC_Cur_State =WAIT_FIRST_CAPTURE;    // to next state,a)wait for first capture
      break;
  }
  TIME_CC_TRG_FLAG = SET;
}

/**
  * @brief System Clock Configuration
  * HSE 8M,if HSE not ready,use HSI instead of HSE,PLL out 48M,EN LSI;
  * @retval None
  */
void SystemClock_Config(void)
{
  MS32_RCC_OscInitTypeDef osc_cfg;
  MS32_RCC_PllInitTypeDef pll_cfg;
  MS32_RCC_ClkInitTypeDef clk_cfg;

  // get and clear reset flag 
  MS32_RCC_GetResetFlag(ENABLE);
  
  // osc config 
  osc_cfg.HSIState = MS32_RCC_HSI_ON;
  osc_cfg.HSEState = MS32_RCC_HSE_ON;
  osc_cfg.LSIState = MS32_RCC_LSI_ON;
  osc_cfg.LSEState = MS32_RCC_LSE_OFF;
  MS32_RCC_OscConfig(&osc_cfg);
  
  // pll config 
  pll_cfg.PLLState  = MS32_RCC_PLL_ON;
  if( MS32_RCC_HSE_IsReady() ) {
    pll_cfg.PLLSource = MS32_RCC_PLL_SOURCE_HSE; // HSE*6 48M  
    pll_cfg.PREDIV    = MS32_RCC_PLL_PREDIV_DIV1;
    pll_cfg.PLLMUL    = MS32_RCC_PLL_MUL6;
  } else {                                       // if HSE not ready,HSI instead of HSE
    pll_cfg.PLLSource = MS32_RCC_PLL_SOURCE_HSI_DIV_2;   
    pll_cfg.PLLMUL    = MS32_RCC_PLL_MUL12;      // HSI/2=4M,*12=48M
  }
  MS32_RCC_PllConfig(&pll_cfg);
  
  // sysclk config 
  if( MS32_RCC_PLL_IsReady() ) {
    clk_cfg.SYSCLKSource   = MS32_RCC_SYSCLK_SOURCE_PLL;
  }  else {
    clk_cfg.SYSCLKSource   = MS32_RCC_SYSCLK_SOURCE_HSI;
  }    

  clk_cfg.AHBCLKDivider  = MS32_RCC_SYSCLK_DIV1;
  clk_cfg.APB1CLKDivider = MS32_RCC_HCLK_DIV1;
  MS32_RCC_ClockConfig(&clk_cfg);
}

/******************************** END OF FILE *********************************/
